/* Array of arbitrary data. The difference with the virtual list is that it
    can be printed, but is slower with a large number of rows.

Example:
    in XUL:
        <html:table id="rl" class="super" flex="1" width="100%" border="1"/>
    in JS:
    {
        var my_header_cb = function(da, td, w)
        {
            td.setAttribute ("width", (w*10)+ "%");
            td.appendChild(document.createTextNode(w));
        };
        var my_content_cb = function(da, td, w, h)
        {
            if (w == 0)
            {
                var cb = document.createElement("checkbox");
                cb.setAttribute("checked", (h%2==1));
                cb.removeAttribute("label");
                td.appendChild(cb);
                return;
            }
            else
            if (w != 2)
            {
                td.appendChild(document.createTextNode("a"+w+"_"+h));
                return:
            }

            var tb = document.createElement("textbox");
            tb.setAttribute("maxlength", 3);
            tb.setAttribute("size", 2);
            td.appendChild(tb);
        };

        var da = new skDataArray(document.getElementById("rl"), "my_data");
        da.setContent(5, 4, my_header_cb, my_content_cb);
    in CSS:
        .my_data_td
        {
            background-color: red;
        }
    }
*/

// `parent' is the containing <html:table> element
// `class_prefix' is the (optional) string that will prefix all classes of the sub-elements.
// (thead, tbody, td, tr). If `class_prefix' is `AAA', then <thead> will have the class
// `AAA_thead'.
// The additional attribute `parity' (`odd' or `even') is set on all <tr> in the <tbody>.
function skDataArray(parent, class_prefix)
{
    if (parent == undefined)
        throw "skDataArray.skDataArray(parent): Undefined parent container";

    this.parent = parent;
    if (class_prefix && class_prefix != "")
        this._class_prefix = class_prefix + "_";
    else
        this._class_prefix = "";
}

skDataArray.prototype =
{
    _set: false,
    _body: null,
    _skip_header: false,
    //class_prefix (string)
    parent: null,

    /* Create the table with `width' columns and `height' rows.
        `cb_header(da,td,w)' has to be specified to add attributes and content to the
            header <td>'s.
            `da' is the pointer to the skDataArray object
            `td' is the pointer to the current <td> object
            `w' is the index of the column

        `cb_data(da,w,h)' has to be specified to add the actual content
            `da' is the pointer to the skDataArray object
            `w' is the index of the column
            `h' is the index of the row
    */
    setContent: function(width, height, cb_header, cb_data)
    {
    try
    {
        const XHTML = "http://www.w3.org/1999/xhtml";

        if (this._set)
            this.clearContent();

        var w;
        if (cb_header != null)
        {
            var th = document.createElementNS(XHTML, "thead");
            th.setAttribute("class", this._class_prefix + "thead");
            this.parent.appendChild(th);

            // create <tr> nodes?
            // var tr = document.createElementNS(XHTML, "tr");

            for (w = 0; w < width; w++)
            {
                td = document.createElementNS(XHTML, "td");
                td.setAttribute("class", this._class_prefix + "td");
                cb_header(this, td, w);

                th.appendChild(td);
            }
            this._skip_header = false;
        }
        else
            this._skip_header = true;

        var tb;
        if (this._body == null)
        {
            tb = document.createElementNS(XHTML, "tbody");
            tb.setAttribute("class", this._class_prefix + "tbody");
            this.parent.appendChild(tb);
            this._body = tb;
        }
        else
            tb = this._body;

        for (var h = 0; h < height; h++)
        {
            tr = document.createElementNS(XHTML, "tr");
            tr.setAttribute("class", this._class_prefix + "tr");
            tr.setAttribute("parity", (h % 2 == 0) ? "even" : "odd");
            tb.appendChild(tr);

            for (w = 0; w < width; w++)
            {
                td = document.createElementNS(XHTML, "td");
                td.setAttribute("class", this._class_prefix + "td");
                tr.appendChild(td);
                if (cb_data)
                    node = cb_data(this, td, w, h);
            }
        }

        this._set = true;
    }
    catch (e)
    {
        throw ("skDataArray.setContent: " + e);
    }
    },
    /* Delete the content of the table */
    clearContent: function()
    {
    try
    {
        for (var c = this.parent.rows.length; c; c--)
            this.parent.deleteRow(c - 1);
        if (!this._skip_header)
            this.parent.deleteTHead();
        if (this._body)
        {
            this.parent.removeChild(this._body);
            this._body = null;
        }
        this.parent.deleteTFoot();
        this._set = false;
    }
    catch (e)
    {
        throw ("skDataArray.clearContent: " + e);
    }
    }
}
