/* *************************************************************************************
   *
   * File : extendString.js                                    $Revision: 1.2 $
   *                                                           $Date: 2005/06/07 10:05:22 $
   *                                                           $Author: tristan $
   *                                                           $Name:  $
   * Purpose :
   *
   * Extends the string object for adding common used string function
   *
   *************************************************************************************
*/

/******
Eleve les espaces de debut et de fin de la chaine de caractere
******/
String.prototype.trim = function (){
    if (!this) return "";
    return this.replace(/(^\s+|\s+$)/g, '')
}

/******
Eleve les espaces du debut de la chaine de caractere
******/
String.prototype.trimLeft = function (){
    if (!this) return "";
    return this.replace(/^\s+/, "");
}

/******
Eleve les espaces de la fin de la chaine de caractere
******/
String.prototype.trimRight = function (){
    if (!this) return "";
    return this.replace(/\s+$/, '');
}

/******
Renvoie true ou false si la chaine est composee d'un ou plusieur espaces
******/
String.prototype.isWhiteSpace = function (){
    if (!this) return "";
    return /^\s+/.test(this);
}

/******
Tronque une chaine
maxLength           ---> taille maxi de la chaine
addEllipses         ---> true ou false pour ajouter '...'
******/
String.prototype.truncateStringAtWordEnd = function(maxLength, addEllipses){
    // Return empty if string is null, undefined, or the empty string
    if (!this) return "";

    // We assume they probably don't want whitespace at the beginning
    var s = this.trimLeft();
    if (s.length <= maxLength) return this;

    // We need to truncate the string to maxLength or fewer chars
    if (addEllipses) maxLength -= 3;

    s = s.replace(RegExp("(.{0," + maxLength + "})\\s.*"), "$1")

    if (s.length > maxLength) s = s.slice(0, maxLength);

    if (addEllipses) s += "...";

    return s;
}


/******
Converti la chaine unicode dans le charset demande
charset       ---> Charset de convertion
******/
String.prototype.convertFromUnicode = function(charset) {
    var ScriptableUnicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance();
    ScriptableUnicodeConverter.QueryInterface(Components.interfaces.nsIScriptableUnicodeConverter);
    ScriptableUnicodeConverter.charset = charset;
    return ScriptableUnicodeConverter.ConvertFromUnicode(this);
}

/******
Converti le charset demande en unicode
charset       ---> Charset de convertion
******/
String.prototype.convertToUnicode = function(charset) {
    try {
        var ScriptableUnicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance();
        ScriptableUnicodeConverter.QueryInterface(Components.interfaces.nsIScriptableUnicodeConverter);
        ScriptableUnicodeConverter.charset = charset;
        return ScriptableUnicodeConverter.ConvertToUnicode(this);
    } catch(err) {
        alert("ERROR function ConvertToUnicode : \n"+err+"\n"+ this);
    }
}

/******
Converti la chaine de caractere en minuscule UTF-8
******/
String.prototype.lowerUTF8 = function () {
    var factory = Components.classes["@idm.fr/sk_factory;3"].getService(Components.interfaces.skXIFactory);
    /* La fonction attend de l'UTF-8 et retourne de l'UTF-8 */
    return factory.SimplifyString(factory.SIMPLIFY_CASE, this.convertFromUnicode("UTF-8")).convertToUnicode("UTF-8");
}

/******
Renvoie la chaine de caractere avec les retours chariot remplaces par des <BR>
******/
String.prototype.replaceCarriageReturnByBR = function () {
    return this.replace(/\r?\n/g,"<BR>")
}

/******
Renvoie true ou false si la chaine est composee de decimal
******/
String.prototype.stringOfDigits = function(){
    return this.testChars("0123456789");
}

/******
Renvoie true ou false si la chaine n'est composee que de la liste des caracteres demande
listChars       ---> chaine de caractere (liste des caracteres a tester)
******/
String.prototype.testChars = function(listChars){
    var nbChar = this.length
    if (nbChar == 0)
        return false;    // blank entry is not a string of digits

    for(var i=0; i<nbChar; i++){
        var thisChar = this.charAt(i);    // this line necessary because of bug (?) (Mac only?)
        if (listChars.indexOf(thisChar) == -1)
            return false;
    }
    return true;
}
