/**
 * Creates an picture dictionary from a group of widgets
 * The Oxford picture dictionary is made of a search box, a radiogroup, 2 indexes and 2 AZ lists
 * For instance:
 *
      <!-- search box -->
      <textbox  id="pictSearch" group="pictSearch" autoscroll="true"/>
      <!-- triggers the search -->
      <button group="pictSearch"/>
      <!-- radiogroup -->
      <radiogroup id="pict-radiogroup" group="pictSearch,vidSearch" orient="horizontal">
          <radio media="pict" selected="true"/>
          <radio media="vid"/>
      </radiogroup>

      <!-- alpha indexes -->
      <skVirtualList id="indexPicture"
                     group="pictSearch"
                     src="chrome://doas/content/iframe/azMD.xml"
                     type="index"
                     fs="img.skn/alpha_index.skn"
                     scrollSpeed="3"
                     flex="1"/>
      <skVirtualList id="indexVideo"
                     group="vidSearch"
                     src="chrome://doas/content/iframe/azMD.xml"
                     type="index"
                     fs="video.skn/alpha_index.skn"
                     scrollSpeed="3"
                     flex="1"/>

      <!-- az list -->
      <skVirtualList id="contentPicture"
                     src="chrome://doas/content/iframe/pict.html"
                     group="pictSearch"
                     type="az"
                     fs="img.skn"
                     flex="1"/>
      <skVirtualList id="contentPicture"
                     src="chrome://doas/content/iframe/pict.html"
                     group="vidSearch"
                     type="az"
                     fs="video.skn"
                     flex="1"/>
 */
 
function PictureSearchSystem(id, opts) {
    // JS inheritance
    this.base = SearchSystem;
    // display a single image per default
    if (opts.singleEntry == null)
        opts.singleEntry = true;
    this.switchDisplayCallback = opts.switchDisplayCallback || null;
    this.base(id, opts);
    var me = this;
    this.initRadioGroup();

    this.indexList.selectFirstItem(true);
    
    this.indexList.selectItemCallback(function(item, obj) {
        try{
            var guidToSelect = skTabManager.getFieldValue(me.fsIdx, item.index, "ID");

            // scroll in the main az list if needed
            if (me.azList)
                me.azList.placeInVlist(guidToSelect, null, null, true);
        } catch (err) {
            jsLog(err)
        }
    });

    this.azList.getContentCallback(function (piIndex, obj) {
        try {
            var entry   = skTabManager.getFile(me.fsAz + "/config.cff", piIndex).GetData().GetWStringData();
            entry = entry.replace(/^\s*<\?xml.+\??>/, '');
            entry = entry.replace(/(<img [^\/>]*)>/, '$1/>'); //useless

            if(this.note)
                entry = this.note(entry);
            return String(entry);
        } catch (err) {
            jsLog("GetEntryContent failed: " + err + " ON piIndex : " + piIndex);
        }
    });
}

PictureSearchSystem.prototype = new SearchSystem;

PictureSearchSystem.prototype.assignWidget = function (widget) {
    try {
        // sacrifices a chicken by a moonless night to the Ancient Ones,
        // to simulate 'super'
        var type = $(widget).attr("type");
        var nodeName = $(widget).nodeName();
        if (!SearchSystem.prototype.assignWidget.call(this, widget)) {
            if (nodeName == "radiogroup") {
                this.radiogroup = widget;
            } else
                return false;
            widget.searchSystem = this;
            return true;
        }
        return true;
    }
    catch (e) {
        throw "Error in TopicSearchSystem.assignWidget, in " +
            "ActivatorSearchSystem " + this.id + ": " + e;
    }
}

PictureSearchSystem.prototype.initRadioGroup = function(widget) {
    try {
        if (!this.radiogroup)
            return;

        var me = this;
        this.radiogroup.addEventListener("RadioStateChange",
                                        function() {
                                            me.switchDisplay(this.selectedItem.getAttribute('media'));
                                        }, true);
    } catch(e) {
        throw "Error in initRadioGroup, in pictureSearchSystem " + this.id + 
            ":" + e;
    }
}

PictureSearchSystem.prototype.resetRadioGroup = function(widget) {
    try {
        if (!this.radiogroup)
            return;
        this.radiogroup.selectedIndex = 0;
    } catch(e) {
        throw "Error in resetRadioGroup, in pictureSearchSystem " + this.id + ":" + e;
    }
}

PictureSearchSystem.prototype.switchDisplay = function(media) {
    try {
        if (!this.switchDisplayCallback)
            return;
        this.switchDisplayCallback(media);
    } catch (err) {
       jsLog("ERROR function switchDisplay:\n" + err);
    }
};
