/*****************************************************************************
* Tototo  > skBasketCollection + skBasket                                    *
*----------------------------------------------------------------------------*
* Its goal is to store on the user machine a part or a whole document of any *
* type (text or not).                                                        *
* skBasketCollection is an 'interface' to create and manage a lot of basket  *
* files at the same time. Physically, a basket is a file stored in the       *
* mozilla user profil directory.                                             *
* A basket (see the skBasket object) got a name and a list of elements.      *
*----------------------------------------------------------------------------*
* (c) IDM. 2006                                                              *
* Project  : Totoro / Plugs                                                  *
* Language : JavaScript                                                      *
* Created  : MikaelLebrun, 02/08/2006                                        *
*****************************************************************************/

/*
 *********************************************************************************************************************************
 ** skBasketCollection ***********************************************************************************************************
 *********************************************************************************************************************************
 */
 
function skBasketCollection(psDirectory, pbPersist) {
    if (!psDirectory || pbPersist == null)
        throw("* skBasketCollection: please specify the basket directory and its persistence");

    this.localfile  = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    this.dirservice = Components.classes['@mozilla.org/file/directory_service;1'].createInstance(Components.interfaces.nsIProperties);
    this.basketdir = null;
    this.init(psDirectory, pbPersist);
    this.basketIndex = this.getBasketInternal("_basket_index_", true);
}

skBasketCollection.prototype = {
    /*
     * Private functions
     */

    init:function(psDirectory, pbPersist) {
        var oProfileFile = this.dirservice.get('ProfD',  Components.interfaces.nsIFile);
        this.localfile.initWithFile(oProfileFile);
        this.localfile.appendRelativePath(psDirectory); //Create the basket directory in user profile

        if (!this.localfile.exists())
            this.localfile.create(this.localfile.DIRECTORY_TYPE, 0700); //The basket directory doesn't exist yet

        this.basketdir = this.localfile;

        if (!pbPersist)
            this.deleteBasketList();
    },

    getBasketFile:function (psBasketName, pbCreate) {
        var oFile     = this.basketdir.clone().QueryInterface(Components.interfaces.nsILocalFile);
        var sFileName = psBasketName + ".txt";
        oFile.appendRelativePath(sFileName);
        
        if (pbCreate && !oFile.exists())
            return this.createNewBasketFile(psBasketName);

        return (oFile.exists() ? oFile : null);
    },

    createNewBasketFile:function (psBasketName) {
        var oFile     = this.basketdir.clone().QueryInterface(Components.interfaces.nsILocalFile);
        var sFileName = psBasketName + ".txt";
        oFile.appendRelativePath(sFileName);

        if (oFile.exists())
            oFile.remove(false);

        oFile = this.basketdir.clone().QueryInterface(Components.interfaces.nsILocalFile);
        oFile.appendRelativePath(sFileName);
        oFile.create(oFile.NORMAL_FILE_TYPE, 0700);
        return oFile;
    },

    getBasketInternal:function(psBasketName, pbCreate) {
        if (!psBasketName || pbCreate == null)
            throw("* skBasketCollection.getBasketInternal(): please specify the basket name and the creation option");

        var oFile   = this.getBasketFile(psBasketName, pbCreate);
        var oBasket = (oFile ? new skBasket(psBasketName, oFile) : null);
        return oBasket;
    },
    
    /*
     * Public functions
     */
        
    getBasket:function(psBasketName, pbCreate) {
        if (!psBasketName || pbCreate == null)
            throw("* skBasketCollection.getBasket(): please specify the basket name and the creation option");
        var sBasketId = this.basketIndex.getItem(psBasketName);
        if (!sBasketId) {
            sBasketId = new Date().getTime();
            this.basketIndex.setItem(psBasketName, sBasketId);
        }
        return this.getBasketInternal(sBasketId, pbCreate); 
    },

    deleteBasket:function(psBasketName) {
        if (!psBasketName)
            throw("* skBasketCollection.deleteBasket(): please specify the basket name");

        var sBasketId = this.basketIndex.getItem(psBasketName);
        if (sBasketId) {
            var oFile = this.getBasketFile(sBasketId, false);
        if (oFile)
            oFile.remove(oFile.PR_TRUE);
            this.basketIndex.removeItem(psBasketName);
        }
    },

    deleteBasketList:function() {
        var aFileList = this.getBasketList();
        var i;

        for (i=0; i<aFileList.length; i++)
            this.deleteBasket(aFileList[i]);
    },
    
    // Returns the file names of all the baskets except the index basket
    getBasketFileNames:function() {
        var oFileList, oFile;
        var aFile;
        var sFileName;
        var iPos;

        oFileList = this.basketdir.directoryEntries;
        aFile     = new Array();

        while(oFileList.hasMoreElements()) {
            oFile = oFileList.getNext().QueryInterface(Components.interfaces.nsIFile);
            if (oFile.isFile()) {
                sFileName = oFile.leafName;
                if (sFileName == this.basketIndex.name) continue;
                iPos      = sFileName.lastIndexOf(".");
                sFileName = (iPos == -1 ? sFileName : sFileName.substring(0, iPos));
                aFile.push(sFileName);
            }
        }

        return aFile;
    },

    getBasketList:function() {
        return this.basketIndex.getKeyList();
    },
}

/*
 *********************************************************************************************************************************
 ** skBasket *********************************************************************************************************************
 *********************************************************************************************************************************
 */
    
function skBasket(psBasketName, poFile) {
    this.istreamfile   = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    this.ostreamfile   = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
    this.istreamscript = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
    this.name          = psBasketName;
    this.file          = poFile;
    this.hash          = new Array();
    this.length        = 0;

    this.CHARTAB       = String.fromCharCode(9);
    this.CHARNEWLINE   = String.fromCharCode(10);
    this.MODE_CREATE   = 0x08;
    this.MODE_WRONLY   = 0x02;
    this.MODE_TRUNCATE = 0x20;

    this.init();
}

skBasket.prototype = {
    /*
     * Private functions
     */

    init:function() {
        var aContent, aLine;
        var sFileContent, sLine, sKey, sContent;
        var i;

        sFileContent = this.getFileContent();
        aContent = sFileContent.split(this.CHARNEWLINE);

        for (i=0; i<aContent.length; i++) {
            sLine = aContent[i];
            aLine = sLine.split(this.CHARTAB);
            
            if (aLine.length) {
                sKey     = aLine[0];
                sContent = aLine[1];
                if (sKey && sContent) {
                    this.hash[sKey] = decodeURIComponent(atob(sContent));
                    this.length++;
                }
            }
        }
    },
    
    getFileContent:function() {
        this.istreamfile.init(this.file, 1, 0, true)
        this.istreamscript.init(this.istreamfile);
        var iFileSize    = this.istreamscript.available();
        var sFileContent = this.istreamscript.read(iFileSize);
        this.istreamscript.close();
        this.istreamfile.close();
        return sFileContent;
    },

    save:function() {
        var sKey, sLine;
        var i;

        this.ostreamfile.init(this.file, this.MODE_WRONLY | this.MODE_CREATE | this.MODE_TRUNCATE, 0664, 0);

        for (sKey in this.hash) {
            sLine = sKey + this.CHARTAB + btoa(encodeURIComponent(this.hash[sKey])) + this.CHARNEWLINE;
            this.ostreamfile.write(sLine, sLine.length);
        }

        this.ostreamfile.flush();
        this.ostreamfile.close();
    },
    
    /*
     * Public functions
     */
    
    getKeyList:function() {
        var aItem = new Array();
        for(var i in this.hash){
            aItem.push(i);
        }
        return aItem;
    },
    getItem:function(psKey) {
        if (!psKey)
            throw("* skBasket.getItem(): please specify the key");

        var sContent = this.hash[new String(psKey)];
        return sContent;
    },
    
    setItem:function(psKey, psValue) {
        if (psKey == undefined || psValue == undefined || psKey == null || psValue == null)
            throw("* skBasket.setItem(): please specify both key and value");

        if (typeof(this.hash[new String(psKey)]) == 'undefined')
            this.length++;
        this.hash[new String(psKey)] = psValue;
        this.save();
    },

    removeItem:function(psKey) {
        if (!psKey)
            throw("* skBasket.removeItem(): please specify psKey");

        if (typeof(this.hash[new String(psKey)]) != 'undefined') {
            delete this.hash[new String(psKey)];
            this.length--;
            this.save();
        }
    },

    getCount:function() {
        return this.length;
    },
    
    empty:function() {
        this.hash = new Array();
        this.save();
    }
}
