/*
History object used to save any type of information (searches / selected documents ...)
*/

function skHistoryManager(basketName, backBtn, nextBtn, showItemCallback) {
    try {
        this.domparser = new DOMParser();
        this.basket    = Components.classes["@idm.fr/sk_basket;1"].createInstance().QueryInterface(Components.interfaces.skIBasket);
        this.basket.SetWindow(window);
        if (!this.basket)
            return;
        this.baskname         = basketName;
        this.backbtn          = backBtn;
        this.nextbtn          = nextBtn;
        this.showItemCallback = showItemCallback;
        this.baskidx          = this.getBasketIndex();
        this.useHistory       = false;
        this.idx              = 0;

        if (this.baskidx == -1) {
            this.basket.AddBasket(this.baskname);
            this.baskidx = this.getBasketIndex();
        } else
            this.basket.EmptyBasket(this.baskidx);

        this.idx = this.basket.BasketContentCount(this.baskidx);
    } catch (err) {
        throw("ERROR in skHistoryManager: " + err);
    }
}

skHistoryManager.prototype.getBasketIndex = function() {
    try {
        return this.basket.GetBasketIndexByName(this.baskname);
    } catch (err) {
        throw("ERROR in skHistoryManager.prototype.getBasketIndex:\n" + err);
    }
}

skHistoryManager.prototype.saveXmlToBasket = function(val) {
    try {
        if (this.useHistory) {
            this.useHistory = false;
            return;
        }

        //Remove elements at the end of the history (if the current position is not the last)
        if (this.idx < this.basket.BasketContentCount(this.baskidx)-1) {
            for (var i=this.basket.BasketContentCount(this.baskidx)-1; i>this.idx; i--)
                this.basket.RemoveElement(this.baskidx, i);
        }
        //Avoid the insertion of duplicated elements
        if ( this.basket.BasketContentCount(this.baskidx) > 0 
             && (val == this.basket.ReadNameAt(this.baskidx, this.basket.BasketContentCount(this.baskidx)-1)) )
            return;

        this.basket.AddContentInBasket("title", val, this.baskidx);
        this.idx = this.basket.BasketContentCount(this.baskidx)-1;
        this.setButtonStates((this.idx > 0), false);
    } catch (err) {
        throw("ERROR in skHistoryManager.prototype.saveXmlToBasket:\n" + err);
    }
}

skHistoryManager.prototype.navigTo = function(increment) {
    try {
        this.idx          = Number(this.idx) + Number(increment);
        this.setButtonStates( (this.idx > 0),
                             (this.idx < this.basket.BasketContentCount(this.baskidx)-1) );
        var element = this.basket.ReadNameAt(this.baskidx, this.idx);
        var xml     = this.domparser.parseFromString(element, "text/xml");
        this.showHistoryItem(xml);
    } catch (err) {
        throw("ERROR in skHistoryManager.prototype.navigTo:\n" + err);
    }
}

skHistoryManager.prototype.setCurrentButtonStates = function() {
    try {
        this.setButtonStates((this.idx > 0),
                            (this.idx < this.basket.BasketContentCount(this.baskidx)-1));
    } catch (err) {
        throw("ERROR in skHistoryManager.prototype.setCurrentButtonStates:\n" + err);
    }
}

skHistoryManager.prototype.setButtonStates = function(enableBackbtn, enableNextBtn) {
    try {
        this.backbtn.disabled = !enableBackbtn;
        this.nextbtn.disabled = !enableNextBtn;
    } catch (err) {
        throw("ERROR in skHistoryManager.prototype.setButtonStates: " + err);
    }
}

skHistoryManager.prototype.showHistoryItem = function(xml) {
    try {
        if (this.showItemCallback) {
            this.useHistory = true;
            this.showItemCallback(xml);
        }
    } catch (err) {
        throw("ERROR in skHistoryManager.prototype.showHistoryItem:\n" + err);
    }
}

skHistoryManager.prototype.getXmlFromBasket = function(increment){
    try {
        var index    = Number(this.idx) + Number(increment);
        var element = this.basket.ReadNameAt(this.baskidx, index);
        return this.domparser.parseFromString(element, "text/xml");
    } catch (err) {
       throw("ERROR in skHistoryManager.prototype.getXmlFromBasket:\n" + err);
    }
}
