function skTabManagerObj(){
    this.init();
}
skTabManagerObj.prototype = {
    init: function (){
        this.skFactory = Components.classes["@idm.fr/sk_factory;3"].
            getService(Components.interfaces.skXIFactory);
        this.skFactory.Init();
        this.skCore = Components.classes["@idm.fr/sk_core_service;3"].
            getService(Components.interfaces.skXICoreService);
        this.dataArray = new Array();
        this.decreg = new RegExp('^[0-9]+$');
        // initialize it to do an action when we can't find the data and the
        // user doesn't want to insert a media containing them
        this.addDiskAbortCallback = null;
    },
    add: function (psPath){
        if(!this.dataArray[psPath]){
            var endPath = psPath.slice(psPath.lastIndexOf("."));
            switch(endPath){
                case ".cfi":
                    this.createIndex(psPath);
                    break;
                case ".cff":
                    this.createFS(psPath);
                    break;
                case ".skn":
                case ".cft":
                    this.createRS(psPath);
                    break;
                default :
                    throw(new Error("Unknown data type :\n  skTabManager.add('"+psPath+"')"));
            }
        }
        return this.dataArray[psPath];
    },

    /* Executes an FS-creating related callback. If the function fails, tries to reexecute
        the callback after a sucessful addDisk. */
    initializePath: function (callback) {
        try
        {
            // Pass 'this' to the callback because JS closures are broken
            return callback(this);
        }
        catch (e)
        {
            var bFailed = true;
            if (!this.cdName) {
                throw "ERROR: Part of the data is missing, and I don't know the name of the media to find the data from";
            }
            while(bFailed) {
                try {
                    this.skFactory.AddDisk(this.cdName, false);
                    return callback(this);
                    bFailed = false;
                } catch(e) {
                    if (this.alertMode) {
                        AlertWithTitle(this.titleBox, this.messageBox);
                        bFailed = false;
                        bAbort = true;
                    } else if (!ConfirmWithTitle(this.titleBox, this.messageBox, this.messageOkBox, this.messageCancelBox)) {
                        bFailed = false;
                        bAbort = true;
                    }
                }
            }
        }
        // execute a callback (if present) when the user doesn't want to insert the media
        // with the missing data
        if (bAbort && this.addDiskAbortCallback) {
            this.addDiskAbortCallback();
        }
    },

    createRS: function (psPath){
        try{
            // Use the me=this trick because JS closures are broken
            var newRS = this.initializePath(
                function(me) { return me.skFactory.CreateInstance("recordset:native:" + psPath) } 
            );
            this.dataArray[psPath] = new Array();
            this.dataArray[psPath]['data'] = newRS;
            this.dataArray[psPath]['type'] = 'RS';
            this.dataArray[psPath]['fields'] = new Array();
            this.dataArray[psPath]['collection'] = newRS.GetFldCollection();
            return this.dataArray[psPath];
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.createRS('"+psPath+"') :\n"+e.toString();
            throw(err);
        }
    },


    createFS: function (psPath){
        try{
            var newFS = Components.classes["@idm.fr/sk_file_system;3"].createInstance();
            this.initializePath( function() { 
                    newFS.SetFileName( psPath );
                } );
            this.dataArray[psPath] = new Array();
            this.dataArray[psPath]['data'] = newFS;
            this.dataArray[psPath]['type'] = 'FS';
            return this.dataArray[psPath];
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.createFS('"+psPath+"') :\n"+e.toString();
            throw(err);
        }
    },
    createIndex: function (psPath, psOperator){
        try{
            psOperator = psOperator == null ? "|(&(!(^|<(" : psOperator;
            var newIndex = Components.classes["@idm.fr/sk_index;3"].createInstance();
            this.initializePath( function() { newIndex.SetFileName( psPath ) } );
            if(psOperator) newIndex.SetOperators(psOperator);
            this.dataArray[psPath] = new Array();
            this.dataArray[psPath]['data'] = newIndex;
            this.dataArray[psPath]['type'] = 'INDEX';
            return this.dataArray[psPath];
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.createIndex('"+psPath+"', '"+psOperator+"') :\n"+e.toString();
            throw(err);
        }
    },
    getField: function(psPath, pField, pbNameOnly){
        try{
            var thisTable = this.add(psPath);
            var field;
            var collection = thisTable['collection'];
            if(!thisTable['fields'][pField] && this.decreg.test(pField)){
                field = collection.GetFieldByPosition(pField);
                pField = field.GetName();
            }
            if(pbNameOnly){
                return pField;
            }
            if(!thisTable['fields'][pField]){
                if(!field) field = collection.GetFieldByName(pField);
                this.dataArray[psPath]['fields'][pField] = field;
            }else{
                field = thisTable['fields'][pField];
            }
            return field;
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.getField('"+psPath+"', '"+pField+"', '"+pbNameOnly+"') :\n"+e.toString();
            throw(err);
        }
    },
    // Returns true if the table given by 'path' has the field 'fieldName'
    hasField: function(path, fieldName) {
        var fieldCollection = this.add(path)['collection'];
        var fieldCount = fieldCollection.GetFieldCount();
        var hasField = false;
        for (var i = 0 ; (i < fieldCount && !hasField) ; i++) {
            hasField = (fieldCollection.GetFieldByPosition(i).GetName() == fieldName);
        }
        return hasField;
    },
    getFieldValue: function (psPath, piRecordId, pField, xiRecord){
        try{
            var field = this.getField(psPath, pField);
            if(!xiRecord)
                xiRecord = this.getSkTable(psPath).GetRecord(piRecordId);
            var returnValue = "";
            switch(field.GetFieldType()){
                case 1: //skFTData
                    returnValue = xiRecord.GetDataFieldValue(field).GetWStringData();
                    break;
                case 2: //skFTUNum
                    returnValue = xiRecord.GetUNumFieldValue(field);
                    break;
                case 3: //skFTSNum
                    returnValue = xiRecord.GetSNumFieldValue(field);
                    break;
                case 4: //skFTLink
                    returnValue = xiRecord.GetLinkFieldValue (field);
                    break;
            }
            return returnValue;
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.getFieldValue('"+psPath+"', '"+piRecordId+"', '"+pField+"', '"+xiRecord+"') :\n"+e.toString();
            throw(err);
        }
    },
    getFieldValuesArray: function (psPath, piRecordId, paField, xiRecord){
        if(!xiRecord)
            xiRecord = this.add(psPath)['data'].GetRecord(piRecordId)
        if(!xiRecord) return [];
        var nbField = paField.length;
        var returnvaluesArray = [];
        for(var i=0; i<nbField; i++){
            returnvaluesArray[i] = 
              this.getFieldValue(psPath, piRecordId, paField[i], xiRecord);
        }
        return returnvaluesArray;
    },
    getCount: function (psPath){
        var aTab = this.add(psPath);
        return (aTab ? aTab['data'].GetCount() : 0);
    },
    getSkTable: function (psPath){
        var aTab = this.add(psPath);
        return (aTab ? aTab['data'] : null);
    },
    // return file title from FS
    getFileTitle: function (psPath, pKey){
        try{
            var file = this.getFile(psPath, pKey);
            var title = file.GetTitle().GetWStringData();
            return title;
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.getFileTitle('"+psPath+"', '"+pKey+"') :\n"+e.toString();
            throw(err);
        }
    },
    // return file path and name from FS
    getFilePath: function (psPath, pKey){
        try{
            var file = this.getFile(psPath, pKey);
            var filename = file.GetName();
            var filepath = file.GetPath();
            var path = filepath+'/'+filename;
            return path;
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.getFilePath('"+psPath+"', "+pKey+") :\n"+e.toString();
            throw(err);
        }
    },
    // return file object from FS
    getFile: function (psPath, pKey){
        try{
            var file;
            if(this.decreg.test(pKey)){
                file = this.getSkTable(psPath).GetFile(pKey);
            }else{
                file = this.getSkTable(psPath).GetFileByPath(pKey);
            }
            return file;
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.getFile('"+psPath+"', "+pKey+") :\n"+e.toString();
            throw(err);
        }
    },
    // return cursor from rs
    extractCursor: function (psPath, pField, piStart, piEnd){
        try{
            var rs = this.getSkTable(psPath);
            var field = this.getField(psPath, pField);
            if(!piEnd){
              piEnd = rs.GetCount();
            }else if(piEnd < piStart){
              jsLog("ERROR in skTabManager.extractCursor('"+psPath+"', '"+pField+"', '"+piStart+"', '"+piEnd+"') :\n'end' index min then 'start' index");
            }
            var cursor = rs.ExtractCursor(field, piStart, piEnd);
            return cursor;
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.extractCursor('"+psPath+"', '"+pField+"', '"+piStart+"', '"+piEnd+"') :\n"+e.toString();
            throw(err);
        }
    },
    execWhileUnavailableTab: function (psPath, psCallback){
        /* If we don't find the fs, we're going to prompt the user to insert the disk
          (that's what the callback should do, anyway)
           And keep bugging him until he either hits cancel or hits ok and then insert the disk
        */
        for (; ; )
        {
            try
            {
                this.getSkTable(psPath);
                return true;
            }
            catch (e)
            {
                if (!eval(psCallback + "();"))
                    return false;
            }
        }

    },
    // return specified sksimplifier
    getSimplifier: function(psType){
        return this.skCore.GetStringSimplifier(psType);
    },
    // return sorting RS
    getSortingTable: function(psPath, pField, piOrder){
        try{
            if(piOrder != 1 && piOrder != -1){
                jsLog("ERROR in skTabManager.getSortingTable('"+psPath+"', '"+pField+"', "+piOrder+") :\n'order' must be 1 or -1");
                return null;
            }
            var thisTable = this.add(psPath);
            var fieldName = this.getField(psPath, pField, true);
            if(thisTable['sorted'] && thisTable['sorted'][fieldName]){
                return thisTable['sorted'][fieldName];
            }else if(!thisTable['sorted']){
                this.dataArray[psPath]['sorted'] = new Array();
            }
            var rs = thisTable['data'];
            var rc;
            switch(this.getField(psPath, pField).GetFieldType()){
                case 1: //skFTData
                    rc = Components.classes["@idm.fr/sk_record_comparator_data_field;3"];
                    rc = rc.createInstance(Components.interfaces.skXIRecordComparatorDataField);
                    break;
                case 2: //skFTUNum
                    rc = Components.classes["@idm.fr/sk_record_comparator_unum_field;3"];
                    rc = rc.createInstance(Components.interfaces.skXIRecordComparatorUNumField);
                    break;
                case 3: //skFTSNum
                    rc = Components.classes["@idm.fr/sk_record_comparator_snum_field;3"];
                    rc = rc.createInstance(Components.interfaces.skXIRecordComparatorSNumField);
                    break;
                case 4: //skFTLink
                    var err = new Error("ERROR in skTabManager.getSortingTable('"+psPath+"', '"+pField+"') :\ncannot sort with link field");
                    throw(err);
                    return null;
            }

            rc.SetField(fieldName);
            rc.SetSortingOrder(piOrder);

            var sorted = rs.SortToNew(rc, false);
            this.dataArray[psPath]['sorted'][fieldName] = sorted;
            return sorted;
        }catch(e){
            var err = e;
            err.message = "ERROR in skTabManager.getSortingTable('"+psPath+"', '"+pField+"', "+piOrder+") :\n"+e.toString();
            throw(err);
        }
    },
    // return id_mapping
    getIdMapping: function(psPath, publisher_id){
        mappingFs = psPath + "/mapping.skn";
        lcFs = mappingFs + "/lc_publisher_id.skn";
        try {
            var thisTable = this.getSkTable(lcFs);
            // retrieve the idm_id matching the publisher id
            var position = thisTable.LookupText(publisher_id, Components.interfaces.skXIRecordSet.lmEXACT);
            var a_id = this.getFieldValue(lcFs, position, "a_id");
            var idm_id = this.getFieldValue(mappingFs, a_id, "idm_id");
            return idm_id;
        }
        catch (e) {
            throw("(fs=" + lcFs + 
                    ", publisher_id=" + publisher_id + ")\n" + e);
        }
        return null;
    }
}

var skTabManager = new skTabManagerObj();


/*******

skTabManager.add(psType, psPath, psOperator) //psOperator -->> pas obligatoire

*******/
