/** This is 'jquery-lite' for XUL */
// Initialization borrowsed from jQuer+
var widget = window.widget = function( selector, context ) {
	// The jQuery object is actually just the init constructor 'enhanced'
	return new widget.prototype.init( selector, context );
};
window.$ = widget;
/**
 * A widget in a group. Can react to different events sent by its group
 */
widget.fn = widget.prototype = {
    init: function (element) {
        this.element = element;
        return this;
    },
    // add a handler for a given event type
    bind: function (eventType, handler) {
        try {
            widget.event.bind(this.element, eventType, handler);
            return this;
        }
        catch(err) {
            jsLog("Error in bind when binding " + eventType + ": " + err);
        }
    },
    click: function (handler) {
        try {
            widget.event.click(this.element, handler);
            return this;
        }
        catch(err) {
            jsLog("Error in click: " + err);
        }
    },
    keypress: function (handler) {
        try {
            widget.event.keypress(this.element, handler);
            return this;
        }
        catch(err) {
            jsLog("Error in keypress: " + err);
        }
    },
    keyup: function (handler) {
        try {
            widget.event.keyup(this.element, handler);
            return this;
        }
        catch(err) {
            jsLog("Error in keyup: " + err);
        }
    },

    // add a CSS class to the widget
    addClass: function(klass) {
        try {
            var classes = this.element.getAttribute("class");
            if (classes != null) {
                classes = classes.replace(/^ +/, '');
                classes = classes.replace(/ +$/, '');
            }
            var splitClasses = classes != null && classes != '' ? classes.split(' ') : [];
            for (var i = 0 ; i < splitClasses.length ; i++) {
                if (splitClasses[i] == klass)
                    return;
            }
            splitClasses.push(klass);
            this.element.setAttribute("class", splitClasses.join(" "));
            return this;
        }
        catch(err) {
            jsLog("Error in addClass when adding class " + klass + ": " + err);
        }
    },

    // remove a CSS class to the widget
    removeClass: function(klass) {
        try {
            var classes = this.element.getAttribute("class");
            if (classes == null)
                return this;
            var splitClasses = classes.split(" ");
            var newClasses = [];
            for (var i = 0 ; i < splitClasses.length ; i++) {
                if (splitClasses[i] != klass)
                    newClasses.push(splitClasses[i]);
            }
            this.element.setAttribute("class", newClasses.join(" "));
            return this;
        }
        catch(err) {
            jsLog("Error in removeClass when removing class " + klass + ": " + err);
        }
    },
    hasClass: function(klass) {
        try {
            var classes = this.element.getAttribute("class");
            if (classes != null) {
                var splitClasses = classes.split(" ");
                for (var i = 0 ; i < splitClasses.length ; i++) {
                    if (splitClasses[i] == klass)
                        return true;
                }
            }
            return false;
        }
        catch(err) {
            jsLog("Error in hasClass when testing for class " + klass + ": " + err);
        }
    },
    toggleClass: function(klass) {
        try {
            if (this.hasClass(klass))
                this.removeClass(klass);
            else
                this.addClass(klass);
            return this;
        }
        catch(err) {
            jsLog("Error in toggleClass when toggling class " + klass + ": " + err);
        }
    },
    // trigger all handlers when receiving an event
    trigger: function(eventType, src) {
        try {
            widget.event.trigger(this.element, eventType, src);
            return this;
        }
        catch(err) {
            jsLog("Error in trigger when triggering " + eventType + ": " + err);
        }
    },
    attr: function(attributeName, attributeValue) {
        try {
            if (attributeValue == null)
                return this.element.getAttribute(attributeName);
            else {
                this.element.setAttribute(attributeName, attributeValue);
                return this;
            }
        }
        catch(err) {
            jsLog("Error in attr for attribute " + attributeName + ": " + err);
        }
    },
    removeAttr: function(attributeName) {
        try {
            this.element.removeAttribute(attributeName);
            return this;
        }
        catch(err) {
            jsLog("Error in removeAttr for attribute " + attributeName + ": " + err);
        }
    },
    hasAttr: function(attributeName) {
        try {
            return this.element.hasAttribute(attributeName);
        }
        catch(err) {
            jsLog("Error in hasAttr for attribute " + attributeName + ": " + err);
        }
    },
    parent: function() {
        try {
            return widget(this.element.parentNode);
        }
        catch(err) {
            jsLog("Error in parent: " + err);
        }
    },
    nodeName: function() {
        try {
            return this.element.nodeName;
        }
        catch(err) {
            jsLog("Error in nodeName: " + err);
        }
    },
    get: function() {
        try {
            return this.element;
        }
        catch (err) {
            jsLog("Error in get: " + err);
        }
    },
}

widget.prototype.init.prototype = widget.prototype;

widget.event = {
    click: function(element, handler) {
       element.addEventListener("click", handler, true);
    },
    keypress: function(element, handler) {
       element.addEventListener("keypress", handler, true);
    },
    keyup: function(element, handler) {
       element.addEventListener("keyup", handler, true);
    },
    bind: function(element, eventType, callback) {
        if (!widget.storage)
            widget.storage = {};
        if (!widget.storage[element])
            widget.storage[element] = {}
        if (!widget.storage[element][eventType])
            widget.storage[element][eventType] = [];
        widget.storage[element][eventType].push(callback);
    },
    trigger: function(element, eventType, src) {
        if (widget.storage[element] && widget.storage[element][eventType]) {
            var callbacks = widget.storage[element][eventType];
            var callback;
            for (var i = 0 ; i < callbacks.length ; i++) {
                callback = callbacks[i];
                callback.apply(element, [src]);
            }
        }
    },
}
