const ANNOTATION_EDIT = 1;
const ANNOTATION_LIST = 2;

/* Annotation window */
function initAnnotationPopup() {
    try {
        var oData          = window.arguments[0];
        var oEditBtn       = document.getElementById("editBtn");
        var oListBtn       = document.getElementById("showlistBtn");
        var oTitleLabel    = document.getElementById("titleeditlabel");
        var oTitleField    = document.getElementById("title");
        var oContentLabel  = document.getElementById("contenteditlabel");
        var oOkBtn         = document.getElementById("okeditbtn");
        var oRemoveBtn     = document.getElementById("removeeditbtn");
        var oCloseEditBtn  = document.getElementById("closeeditbtn");
        var oListContainer = document.getElementById("annotationListContainer");
        var oCloseListBtn  = document.getElementById("closelistbtn");
        var oDescription;
        var sTitle;

        oData.annotationmanager.openDialog = window;

        //Resize the window
        if (oData.popupwidth && oData.popupheight)
            window.resizeTo(oData.popupwidth, oData.popupheight);

        //Set the labels of navigation buttons (radio buttons 'Edit' and 'List')
        oEditBtn.setAttribute("label", oData.editradiobtn);
        oListBtn.setAttribute("label", oData.listradiobtn);

        //Set tooltips of tabs and buttons
        if (oData.edittooltip)
            oEditBtn.setAttribute("tooltiptext", oData.edittooltip);
        if (oData.listtooltip)
            oListBtn.setAttribute("tooltiptext", oData.listtooltip);
        if (oData.oktooltip)
            oOkBtn.setAttribute("tooltiptext", oData.oktooltip);
        if (oData.removetooltip)
            oRemoveBtn.setAttribute("tooltiptext", oData.removetooltip);

        //Show/Hide the buttons/sections 'Edit'/'Show list'
        showPopupSection(oData.annotationmanager.defaulttab);

        //Load the form
        oTitleLabel.setAttribute("value", oData.titleeditlabel);
        oContentLabel.setAttribute("value", oData.contenteditlabel);
        oTitleField.disabled = oData.disabletextbox;
        oOkBtn.setAttribute("label", oData.okeditbtn);
        oRemoveBtn.setAttribute("label", oData.removeeditbtn);
        oCloseEditBtn.setAttribute("label", oData.closeeditbtn);

        //Load the list of annotations
        var aMyArray = oData.annotationlist;
        if(aMyArray.length>1) aMyArray.sort(sort_alpha);
        for (var j=0; j<aMyArray.length; j++) {
            sTitle = aMyArray[j].title;

            if (sTitle=="")
                continue;
            oDescription = document.createElement("label");
            oDescription.setAttribute("idx", aMyArray[j].idx);
            if(oData.annotationmanager.bXml){
                var oTitle = oData.annotationmanager.oDomParser.parseFromString(convertToUnicode("UTF-8", sTitle), "text/xml");
                oDescription.appendChild(oTitle.firstChild);
            }else{
                oDescription.setAttribute("value", sTitle);
            }
            oDescription.setAttribute("class", "skAnnotationViewLabel");
            oListContainer.firstChild.appendChild(oDescription);
        }
        oCloseListBtn.setAttribute("label", oData.closelistbtn);
        buttStatus();
    } catch (err) {
       alert("ERROR function initAnnotationPopup:\n" + err);
    }
}
function convertToUnicode(charset, sChaine) {
    try {
        var ScriptableUnicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance();
        ScriptableUnicodeConverter.QueryInterface(Components.interfaces.nsIScriptableUnicodeConverter);
        ScriptableUnicodeConverter.charset = charset;
        return ScriptableUnicodeConverter.ConvertToUnicode(sChaine);
    } catch(err) {
        alert("ERROR function ConvertToUnicode : \n"+err+"\n"+ sChaine);
    }
}
function convertFromUnicode(charset, sChaine) {
    var ScriptableUnicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance();
    ScriptableUnicodeConverter.QueryInterface(Components.interfaces.nsIScriptableUnicodeConverter);
    ScriptableUnicodeConverter.charset = charset;
    return ScriptableUnicodeConverter.ConvertFromUnicode(sChaine);
}
function sort_alpha(a,b){

    if (a.title.toLowerCase().replace(/<[^>]*>/g, "")==b.title.toLowerCase().replace(/<[^>]*>/g, "")) return 0;
    if (a.title.toLowerCase().replace(/<[^>]*>/g, "")<b.title.toLowerCase().replace(/<[^>]*>/g, "")) return -1;
    return 1;
}

function showPopupSection(piSection) {
    try {
        var oData          = window.arguments[0];
        var oBtnRadiogroup = document.getElementById("btnRadiogroup");
        var oEditBtn       = document.getElementById("editBtn");
        var oListBtn       = document.getElementById("showlistBtn");
        var oRemoveBtn     = document.getElementById("removeeditbtn");
    	var oEditContainer = document.getElementById("annotationEditContainer");
    	var oListContainer = document.getElementById("annotationListContainer");
        var iSection       = (piSection ? piSection : (oData.annotationmanager.curIdx ? null : ANNOTATION_LIST));

        if (iSection) {
            oEditContainer.collapsed = (iSection != ANNOTATION_EDIT);
            oListContainer.collapsed = (iSection != ANNOTATION_LIST);
        } else {
            oEditContainer.collapsed = !(oData.annotationmanager.curIdx && !oData.hasannotation);
            oListContainer.collapsed = (oData.annotationmanager.curIdx && !oData.hasannotation);
        }

        //No selected entry
        if (!oData.annotationmanager.curIdx) {
            oEditContainer.collapsed = true;
            oListContainer.collapsed = false;
            oEditBtn.setAttribute("disabled", "true");
        } else {
            oEditBtn.removeAttribute("disabled");
            //Enable/disable the remove button if the current entry get/or not an annotation
            oRemoveBtn.disabled = (oData.hasannotation ? false : true);
        }

        oBtnRadiogroup.selectedIndex = (oEditContainer.collapsed ? 1 : 0);
    } catch (err) {
       alert("ERROR function showPopupSection:\n" + err);
    }
}

function validEditorForm() {
    try {
        var oTitleField   = document.getElementById("title");
        var oContentField = document.getElementById("content");
        var oData         = window.arguments[0];

        //Not check the title field if used as label (ie disabled)
        if (!oData.disabletextbox && oTitleField.value == "") {
            alert(oData.prompteditmsg);
            oTitleField.focus();
            return;
        }

        if (oContentField.value == "") {
            alert(oData.prompteditmsg);
            oContentField.focus();
            return;
        }

        oData.save    = true;
        oData.title   = oTitleField.value;
        oData.content = oContentField.value;
        window.close();
    } catch(err) {
        alert("ERROR function validEditorForm: " + err);
    }
}

function removeAnnotation() {
    try {
        var oData    = window.arguments[0];
        oData.remove = true;
        window.close();
    } catch (err) {
       alert("ERROR function removeAnnotation:\n" + err);
    }
}


function buttStatus(){
    var oTitleField    = document.getElementById("title");
    var oContentField  = document.getElementById("content");
    var oOkEditBtn  = document.getElementById("okeditbtn");
    if(oTitleField.value=="" || oContentField.value==""){
        oOkEditBtn.setAttribute("disable", "true");
    }else{
        oOkEditBtn.removeAttribute("disable");
    }
}